import numpy as np
import matplotlib as mpl
import matplotlib.pyplot as plt

def ticks(y, pos):
    return "$e^{%.0f}$"%np.log(y)

def proba(x):
  return np.maximum(0, np.minimum(1, np.log(x)))

def price(x):
  return (x>1) * np.minimum((np.e-1+1/np.e)/x, np.log(x)-(1-1/x)/x)

def result(val, V):
  V = sorted(V, reverse=True)
  while V[-1] < V[0]/np.e:
    V.pop()
  
  res = []
  for i,v in enumerate(V):
    pr = 1/((i+1)*(i+2))
    label="\pi_%d > \pi_i"%(i+1)
    if i > 0: label += ">" + ",\;".join(
      "\pi_%d"%(j+1) for j in range(i))
    res.append((pr, proba(val/v), price(val/v), label))
  pr = 1/(len(V)+1)
  label = "\pi_i > " + ",\;".join(
    "\pi_%d"%(j+1) for j in range(len(V)))
  res.append((pr, proba(val/V[0]*np.e), price(val/V[0]*np.e), label))

  return res
  

V = np.exp(np.linspace(-.5, 0, 4))
val = np.linspace(np.exp(-1.5), np.exp(1.5), 1000)

plt.figure(figsize=(8,4), tight_layout=True)
plt.rcParams.update({
    'font.size': 11,
    "text.usetex": True,
    'text.latex.preamble': r'\usepackage{amsmath,amsfonts}'
})

Y = sum(pr*x for pr,x,_,_ in result(val,V))
plt.plot(val, Y, label="$x_i = \mathbb E[c_i]$")
plt.fill_between(val, Y, [0]*len(val), alpha=0.2)

for pr,x,p,l in result(val, V):
  plt.plot(val, x, label="$\mathbb E[c_i\;|\; "+l+"]$")
  
for v in V:
  plt.axvline(v, color="gray", linestyle="dotted")

plt.xscale("log", base=np.e)
plt.gca().xaxis.set_major_formatter(
  mpl.ticker.FuncFormatter(ticks))

plt.legend()
plt.xlabel("$v_i(s)$")

"""
Y = val*sum(pr*p for pr,x,p,_ in result(val,V))
plt.plot(val, Y, label="price")
plt.fill_between(val, Y, [0]*len(val), alpha=0.2)

for pr,x,p,l in result(val, V):
  plt.plot(val, val*p)

for v in V:
  plt.axvline(v, color="gray", linestyle="dotted")
  #plt.axvline(v*np.e, color="gray", linestyle="dotted")

plt.xscale("log", base=np.e)
plt.gca().xaxis.set_major_formatter(
  mpl.ticker.FuncFormatter(ticks))

plt.legend()
plt.xlabel("$v_i(s)$")
"""

plt.savefig("plot-constant.pdf")
plt.show()
